package eu.cloudtm.wpm.rhq.manager;

import org.infinispan.Cache;
import org.infinispan.config.Configuration;
import org.infinispan.config.GlobalConfiguration;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.manager.EmbeddedCacheManager;
import org.rhq.core.domain.measurement.MeasurementDataNumeric;
import org.rhq.core.domain.measurement.MeasurementScheduleRequest;
import org.rhq.core.domain.measurement.calltime.CallTimeData;


import java.util.StringTokenizer;
import java.util.Date;

/**
 *  WPM Manager
 *
 *  @author peluso@gsd.inesc-id.pt , peluso@dis.uniroma1.it
 */
public class WPMManager {


    private static boolean DEBUG = false;

    private static boolean SIMULATE_LOG_SERVICE = false;

    private static WPMManager manager = null;

    private static Object mutex = new Object();

    private Cache cache = null;

    private EmbeddedCacheManager cacheManager = null;

    private WPMManager(){

       if(!DEBUG){
          GlobalConfiguration gc = GlobalConfiguration.getClusteredDefault();
          gc.setClusterName("LogServiceConnection");
          Configuration c = new Configuration();
          c.setCacheMode(Configuration.CacheMode.REPL_SYNC);

          EmbeddedCacheManager cm = new DefaultCacheManager(gc, c);

          this.cacheManager = cm;
          this.cache = cm.getCache();

          if(SIMULATE_LOG_SERVICE){
              //Memory
              this.cache.put("192.168.0.18",""+System.nanoTime()+";"+"MemoryInfo.free:1000000000"+";"+"MemoryInfo.used:3000000000");
              this.cache.put("192.168.0.35",""+System.nanoTime()+";"+"MemoryInfo.free:1000000000"+";"+"MemoryInfo.used:3000000000");

              //CPU
              this.cache.put("192.168.0.18:0",""+System.nanoTime()+";"+"CpuPerc.idle:0.1"+";"+"CpuPerc.sys:0.4"+";"+"CpuPerc.user:0.5");
              this.cache.put("192.168.0.18:1",""+System.nanoTime()+";"+"CpuPerc.idle:0.1"+";"+"CpuPerc.sys:0.4"+";"+"CpuPerc.user:0.5");
              this.cache.put("192.168.0.18:2",""+System.nanoTime()+";"+"CpuPerc.idle:0.1"+";"+"CpuPerc.sys:0.4"+";"+"CpuPerc.user:0.5");
              this.cache.put("192.168.0.18:3",""+System.nanoTime()+";"+"CpuPerc.idle:0.1"+";"+"CpuPerc.sys:0.4"+";"+"CpuPerc.user:0.5");

              this.cache.put("192.168.0.35:0",""+System.nanoTime()+";"+"CpuPerc.idle:0.1"+";"+"CpuPerc.sys:0.4"+";"+"CpuPerc.user:0.5");
              this.cache.put("192.168.0.35:1",""+System.nanoTime()+";"+"CpuPerc.idle:0.1"+";"+"CpuPerc.sys:0.4"+";"+"CpuPerc.user:0.5");

              //FS
              this.cache.put("192.168.0.18:/",""+System.nanoTime()+";"+"fileSystemUsage.usePercent:0.6"+";"+"fileSystemUsage.freePercent:0.4");
              this.cache.put("192.168.0.35:/",""+System.nanoTime()+";"+"fileSystemUsage.usePercent:0.6"+";"+"fileSystemUsage.freePercent:0.4");

              //Infinispan
              this.cache.put("192.168.0.18:Infinispan Cache ( CloudTM )",""+System.nanoTime()+";"+"Rtt:1"+";"+
                      "ApplicationContentionFactor:0.01"+";"+
                      "LocalContentionProbability:0.01"+";"+
                      "MaxReplayTime:5"+";"+
                      "AvgReplayTime:5"+";"+
                      "TransactionsArrivalRate:30"+";"+
                      "LocksAcquisitionRate:10"+";"+
                      "PerTransactionAcquiredLocks:20"+";"+
                      "CommitCommandCost:1"+";"+
                      "RollbackCommandCost:2"+";"+
                      "LockWaitingTime:100"+";"+
                      "CommittedTransactionsWritePercentage:0.2"+";"+
                      "TransactionsWritePercentage:0.2"+";"+
                      "WriteTransactionTotalExecutionTime:1200000"+";"+
                      "ReadOnlyTransactionExecutionTime:3000"+";"+
                      "WriteTransactionLocalExecutionTime:10000"+";"+
                      "LocalExecNoCont:5000"+";"+
                      "PrepareCommandSize:1024"+";"+
                      "CommitCommandSize:300"+";"+
                      "Throughput:30"+";"+
                      "HoldTime:80"+";"+
                      "ClusteredGetCommandSize:30"+";"+
                      "NumNodesInvolvedInPrepare:3"+";"+
                      "TimeoutExceptionOnPrepare:20"+";"+
                      "DeadlockExceptionOnPrepare:50"+";"+
                      "RemoteGetCost:2"+";"+
                      "99Percentile:300"+";"+
                      "95Percentile:300"+";"+
                      "90Percentile:300"+";"+
                      "TopKLocalRead:<key-1=20<key-317=20<key-3478=19<key-27=10<key-8=43"+";"+
                      "TopKRemoteRead:<key-45=20<key-4=43<key-89=19<key-90=43<key-1=21"+";"+
                      "TopKLocalWrite:<key-345=20<key-5=2<key-7=19<key-98=65<key-90=32"+";"+
                      "TopKRemoteWrite:<key-345=20<key-7=20<key-12=19<key-32=76<key-46=9"+";"+
                      "TopKLocked:<key-534=20<key-435=87<key-9023=19<key-126=9<key-5=76"+";"+
                      "TopKContended:<key-67=20<key-435=43<key-3478=19<key-45=6<key-7=7"+";"+
                      //"top-k-timeout:<key-8=20<key-317=32<key-4=43<key-27=32<key-6=54"+";"+
                      //"top-k-deadlock:<key-456=20<key-45=76<key-3478=5<key-5=42<key-8=32"
                      "TopKFailedLock:<key-456=20<key-45=76<key-3478=5<key-5=42<key-8=32"
              );
              this.cache.put("192.168.0.35:Infinispan Cache ( CloudTM )",""+System.nanoTime()+";"+"Rtt:1"+";"+
                      "ApplicationContentionFactor:0.01"+";"+
                      "LocalContentionProbability:0.01"+";"+
                      "MaxReplayTime:5"+";"+
                      "AvgReplayTime:5"+";"+
                      "TransactionsArrivalRate:30"+";"+
                      "LocksAcquisitionRate:10"+";"+
                      "PerTransactionAcquiredLocks:20"+";"+
                      "CommitCommandCost:1"+";"+
                      "RollbackCommandCost:2"+";"+
                      "LockWaitingTime:100"+";"+
                      "CommittedTransactionsWritePercentage:0.2"+";"+
                      "TransactionsWritePercentage:0.2"+";"+
                      "WriteTransactionTotalExecutionTime:1200000"+";"+
                      "ReadOnlyTransactionExecutionTime:3000"+";"+
                      "WriteTransactionLocalExecutionTime:10000"+";"+
                      "LocalExecNoCont:5000"+";"+
                      "PrepareCommandSize:1024"+";"+
                      "CommitCommandSize:300"+";"+
                      "Throughput:30"+";"+
                      "HoldTime:80"+";"+
                      "ClusteredGetCommandSize:30"+";"+
                      "NumNodesInvolvedInPrepare:3"+";"+
                      "TimeoutExceptionOnPrepare:20"+";"+
                      "DeadlockExceptionOnPrepare:50"+";"+
                      "RemoteGetCost:2"+";"+
                      "99Percentile:300"+";"+
                      "95Percentile:300"+";"+
                      "90Percentile:300"+";"+
                      "TopKLocalRead:<key-1=20<key-317=20<key-3478=19<key-27=10<key-8=43"+";"+
                      "TopKRemoteRead:<"+";"+
                      "TopKLocalWrite:<key-345=20<key-5=2<key-7=19<key-98=65<key-90=32"+";"+
                      "TopKRemoteWrite:<"+";"+
                      "TopKLocked:<key-534=20<key-435=87<key-9023=19<key-126=9<key-5=76"+";"+
                      "TopKContended:<key-67=20<key-435=43<key-3478=19<key-45=6<key-7=7"+";"+
                      //"top-k-timeout:<key-8=20<key-317=32<key-4=43<key-27=32<key-6=54"+";"+
                      //"top-k-deadlock:<"
                      "TopKFailedLock:<key-456=20<key-45=76<key-3478=5<key-5=42<key-8=32"
              );

              //Network
              this.cache.put("192.168.0.18:en0",""+System.nanoTime()+";"+"receivedBytes:120000"+";"+"receivedBytesPerSecond:20000000"+";"+"transmittedBytes:120000"+";"+"transmittedBytesPerSecond:1000000");
              this.cache.put("192.168.0.18:en1",""+System.nanoTime()+";"+"receivedBytes:120000"+";"+"receivedBytesPerSecond:20000000"+";"+"transmittedBytes:120000"+";"+"transmittedBytesPerSecond:1000000");
              this.cache.put("192.168.0.18:en2",""+System.nanoTime()+";"+"receivedBytes:120000"+";"+"receivedBytesPerSecond:20000000"+";"+"transmittedBytes:120000"+";"+"transmittedBytesPerSecond:1000000");

              this.cache.put("192.168.0.35:en0",""+System.nanoTime()+";"+"receivedBytes:120000"+";"+"receivedBytesPerSecond:20000000"+";"+"transmittedBytes:120000"+";"+"transmittedBytesPerSecond:1000000");
              this.cache.put("192.168.0.35:en1",""+System.nanoTime()+";"+"receivedBytes:120000"+";"+"receivedBytesPerSecond:20000000"+";"+"transmittedBytes:120000"+";"+"transmittedBytesPerSecond:1000000");
              this.cache.put("192.168.0.35:en2",""+System.nanoTime()+";"+"receivedBytes:120000"+";"+"receivedBytesPerSecond:20000000"+";"+"transmittedBytes:120000"+";"+"transmittedBytesPerSecond:1000000");

          }
       }
    }

    public static WPMManager getOrCreateWPMManager(){



        synchronized (mutex){

            if(manager == null){
                manager = new WPMManager();

            }



        }


        return manager;

    }

    public static void removeWPMManager(){



        synchronized (mutex){

            manager = null;



        }




    }


    public MeasurementDataNumeric getPlatformMeasurementDataNumeric(String platformKey, MeasurementScheduleRequest req){

        MeasurementDataNumeric result = null;

        if(DEBUG){
           if (req.getName().equals("MemoryInfo.free")) {
                 result = new MeasurementDataNumeric(req, Double.valueOf(1000000000.0D));

            }
            else if (req.getName().equals("MemoryInfo.used")) {
                 result = new MeasurementDataNumeric(req, Double.valueOf(3000000000.0D));

            }
        }
        else{
             if(this.cache != null){
                String value = (String) this.cache.get(platformKey);

                if(value!=null){
                    result = extractMeasurementDataNumeric(req, value);
                }
             }
        }


        return result;


    }

    public MeasurementDataNumeric getServiceMeasurementDataNumeric(String platformKey, String componentKey, MeasurementScheduleRequest req){

        MeasurementDataNumeric result = null;

        if(DEBUG){
            if (req.getName().equals("CpuPerc.idle")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(0.10D));

            }
            else if (req.getName().equals("CpuPerc.sys")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(0.40D));

            }
            else if (req.getName().equals("CpuPerc.user")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(0.50D));

            }
            else if (req.getName().equals("fileSystemUsage.usePercent")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(0.60D));

            }
            else if (req.getName().equals("fileSystemUsage.freePercent")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(0.40D));

            }
            else if (req.getName().equals("LockManager.NumberOfLocksAvailable")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(100));

            }
            else if (req.getName().equals("receivedBytes")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(120000.0D));

            }
            else if (req.getName().equals("receivedBytesPerSecond")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(20000000.0D));

            }
            else if (req.getName().equals("transmittedBytes")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(120000.0D));

            }
            else if (req.getName().equals("transmittedBytesPerSecond")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(1000000.0D));

            }
            else if (req.getName().equals("Rtt")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(1.0D));

            }
            else if (req.getName().equals("ApplicationContentionFactor")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(0.01D));

            }
            else if (req.getName().equals("LocalContentionProbability")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(0.01D));

            }
            else if (req.getName().equals("MaxReplayTime")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(5.0D));

            }
            else if (req.getName().equals("AvgReplayTime")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(5.0D));

            }
            else if (req.getName().equals("TransactionsArrivalRate")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(30.0D));

            }
            else if (req.getName().equals("LocksAcquisitionRate")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(10.0D));

            }
            else if (req.getName().equals("PerTransactionAcquiredLocks")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(20.0D));

            }
            else if (req.getName().equals("CommitCommandCost")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(1.0D));

            }
            else if (req.getName().equals("RollbackCommandCost")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(2.0D));

            }
            else if (req.getName().equals("LockWaitingTime")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(100.0D));

            }
            else if (req.getName().equals("CommittedTransactionsWritePercentage")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(0.2D));

            }
            else if (req.getName().equals("TransactionsWritePercentage")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(0.2D));

            }
            else if (req.getName().equals("WriteTransactionTotalExecutionTime")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(1200000.0D));

            }
            else if (req.getName().equals("ReadOnlyTransactionExecutionTime")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(3000.0D));

            }
            else if (req.getName().equals("WriteTransactionLocalExecutionTime")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(10000.0D));

            }
            else if (req.getName().equals("LocalExecNoCont")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(5000.0D));

            }
            else if (req.getName().equals("PrepareCommandSize")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(1024.0D));

            }
            else if (req.getName().equals("CommitCommandSize")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(300.0D));

            }
            else if (req.getName().equals("Throughput")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(30.0D));

            }
            else if (req.getName().equals("HoldTime")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(80.0D));

            }
            else if (req.getName().equals("ClusteredGetCommandSize")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(30.0D));

            }
            else if (req.getName().equals("NumNodesInvolvedInPrepare")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(3.0D));

            }
            else if (req.getName().equals("TimeoutExceptionOnPrepare")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(20.0D));

            }
            else if (req.getName().equals("DeadlockExceptionOnPrepare")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(50.0D));

            }
            else if (req.getName().equals("RemoteGetCost")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(2.0D));

            }
            else if (req.getName().equals("99Percentile")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(300.0D));

            }
            else if (req.getName().equals("95Percentile")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(300.0D));

            }
            else if (req.getName().equals("90Percentile")) {
                result = new MeasurementDataNumeric(req, Double.valueOf(300.0D));

            }

        }
        else{
            if(this.cache != null){
                String value = (String) this.cache.get(platformKey+":"+componentKey);

                if(value!=null){
                    result = extractMeasurementDataNumeric(req, value);
                }
            }
        }


        return result;

    }


    private  MeasurementDataNumeric extractMeasurementDataNumeric(MeasurementScheduleRequest req, String input){

         MeasurementDataNumeric result = null;

         StringTokenizer strTok = new StringTokenizer(input, ";");

         StringTokenizer internalStrTok = null;

         String extractedName = null;

         String extractedValue = null;

         String internalString = null;

         boolean timestampFound = false;

         long timestamp = -1;

         while(strTok.hasMoreTokens()){

             if(!timestampFound){

                 timestamp = Long.parseLong(strTok.nextToken());

                 timestampFound = true;
             }
             else{
                 internalString = strTok.nextToken();
                 internalStrTok = new StringTokenizer(internalString, ":");

                 if(internalStrTok.hasMoreTokens()){
                     extractedName = internalStrTok.nextToken();

                     if(req.getName().equals(extractedName) && internalStrTok.hasMoreTokens()){
                         extractedValue = internalStrTok.nextToken();
						
						 if(!DEBUG && !SIMULATE_LOG_SERVICE){
							if(timestampFound){
								result =  new MeasurementDataNumeric(timestamp, req, Double.valueOf(extractedValue));
							}	
						 }
						 else{
							result =  new MeasurementDataNumeric(req, Double.valueOf(extractedValue));
						 }

                         break;
                     }
                 }
             }

         }


        return result;
    }

    private  CallTimeData extractTopKData(MeasurementScheduleRequest req, String resource, String input){

        CallTimeData result = null;

        StringTokenizer strTok = new StringTokenizer(input, ";");

        StringTokenizer internalStrTok = null;

        String extractedName = null;

        String extractedValue = null;

        String internalString = null;

        boolean timestampFound = false;

        long timestamp = -1;

        while(strTok.hasMoreTokens()){

            if(!timestampFound){

                timestamp = Long.parseLong(strTok.nextToken());

                timestampFound = true;
            }
            else{
                internalString = strTok.nextToken();
                internalStrTok = new StringTokenizer(internalString, ":");

                if(internalStrTok.hasMoreTokens()){
                    extractedName = internalStrTok.nextToken();

                    if(resource.equals(extractedName) && internalStrTok.hasMoreTokens()){
                        extractedValue = internalStrTok.nextToken();

                        if(isTopKData(extractedValue)) {

                            result = createCallTimeData(req, extractedValue);
                        }

                        break;
                    }
                }
            }

        }


        return result;
    }

    private boolean isTopKData(String data) {
        return data.startsWith("<");
    }

    private MeasurementDataNumeric createMeasurementDataNumeric(MeasurementScheduleRequest req, String value) {
        return new MeasurementDataNumeric(req, Double.valueOf(value));
    }

    //list of <key=value<key=value(...)<key=value
    private CallTimeData createCallTimeData(MeasurementScheduleRequest req, String list) {
        StringTokenizer pairTokens = new StringTokenizer(list, "<");
        CallTimeData ctd = new CallTimeData(req);
        Date dummyDate = new Date();

        while(pairTokens.hasMoreTokens()) {
            String pair = pairTokens.nextToken();
            if(pair.equals("")) {
                continue;
            }
            String[] keyAndValue = pair.split("=");
            ctd.addAggregatedCallData(keyAndValue[0], dummyDate, dummyDate, 0, 0, 0, Long.parseLong(keyAndValue[1]));
        }

        return ctd;
    }

    public CallTimeData getTopKeyData(String platformKey, String componentKey, MeasurementScheduleRequest req, String resourceKey) {
        CallTimeData result = null;
        if(DEBUG) {
            if(resourceKey.equals("TopKLocalRead")) {
                result = createCallTimeData(req, "<key-1=20<key-317=20<key-3478=19<key-27=10<key-8=43");
            } else if(resourceKey.equals("TopKRemoteRead")) {
                result = createCallTimeData(req, "<key-45=20<key-4=43<key-89=19<key-90=43<key-1=21");
            } else if(resourceKey.equals("TopKLocalWrite")) {
                result = createCallTimeData(req, "<key-345=20<key-5=2<key-7=19<key-98=65<key-90=32");
            } else if(resourceKey.equals("TopKRemoteWrite")) {
                result = createCallTimeData(req, "<");
            } else if(resourceKey.equals("TopKLocked")) {
                result = createCallTimeData(req, "<key-534=20<key-435=87<key-9023=19<key-126=9<key-5=76");
            } else if(resourceKey.equals("TopKContended")) {
                result = createCallTimeData(req, "<key-67=20<key-435=43<key-3478=19<key-45=6<key-7=7");
            } else if(resourceKey.equals("TopKFailedLock")) {
                result = createCallTimeData(req, "<key-8=20<key-317=32<key-4=43<key-27=32<key-6=54");
            }
        } else{
            if(this.cache != null){
                String value = (String) this.cache.get(platformKey+":"+componentKey);

                if(value!=null){
                    result = extractTopKData(req, resourceKey, value);
                }
            }
        }

        return result;
    }
}
