package eu.cloudtm.wpm.producer;


import java.io.FileInputStream;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.util.Properties;

import eu.cloudtm.resources.MonitorableResources;
import eu.reservoir.monitoring.appl.DynamicControl;
import eu.reservoir.monitoring.core.Probe;
import eu.reservoir.monitoring.core.Rational;
import eu.reservoir.monitoring.distribution.udp.UDPDataPlaneProducer;
import eu.reservoir.monitoring.im.dht.DHTInfoPlane;

public class ResourcesController extends DynamicControl {
	// the start time
    private long startTime = 0;
    private String cpuComponentID;
    private String memComponentID;
    private String dskComponentID;
    private String netComponentID;
    private String jmxComponentID;
    private String IP_Address;
    private String groupId;
    private String providerId;

    // the DataSource
    private ResourcesDataSource dataSource;

    public ResourcesController(String consumer_DP_addr, int consumer_DP_port, String consumer_IP_addr, int consumer_IP_local_port, int consumer_IP_remote_port,String hostname) {
		super("Probes Controller");
		//set up counter
		//set up data source
		InetSocketAddress address = new InetSocketAddress(consumer_DP_addr, consumer_DP_port);
		
		// set up multicast addresses
		//MulticastAddress dataGroup = new MulticastAddress(addr, port);
		//dataSource.setDataPlane(new MulticastDataPlaneProducer(dataGroup));
		//without attributes
		//dataSource.setDataPlane(new UDPDataPlaneProducerNoNames(address));
		//with attributes
		boolean isConnected = true;
		dataSource = new ResourcesDataSource(hostname);
		dataSource.setDataPlane(new UDPDataPlaneProducer(address));
		dataSource.setInfoPlane(new DHTInfoPlane(consumer_IP_addr, consumer_IP_remote_port, consumer_IP_local_port));
		do{
			isConnected = dataSource.connect();
			if(!isConnected){
				System.out.println("Waiting for Consumer Process...");
			}
		}while(!isConnected);
		
    }

    /**
     * Initialize.
     */
    protected void controlInitialize() {
    	System.out.println("Inizializing control plane");
    	startTime = System.currentTimeMillis();
    	loadParametersFromRegistry();
    }

    /**
     * Actually evaluate something.
     */
    protected void controlEvaluate() {
		long now = System.currentTimeMillis();
		long diff = (now - startTime) / 1000;
		System.out.println(diff + ": " + this + " seen " + dataSource.getProbes().size());
		
		if(dataSource.getProbes().size() == 0){
			dataSource.addProbe(cpuComponentID+":"+IP_Address+":"+groupId+":"+providerId,180000,MonitorableResources.CPU);
			dataSource.addProbe(memComponentID+":"+IP_Address+":"+groupId+":"+providerId,180000,MonitorableResources.MEMORY);
			dataSource.addProbe(netComponentID+":"+IP_Address+":"+groupId+":"+providerId,180000,MonitorableResources.NETWORK);
			dataSource.addProbe(dskComponentID+":"+IP_Address+":"+groupId+":"+providerId,180000,MonitorableResources.DISK);
			dataSource.addProbe(jmxComponentID+":"+IP_Address+":"+groupId+":"+providerId,15000,MonitorableResources.JMX);
		}
    }

    /**
     * Cleanup
     */
    protected void controlCleanup() {
    	System.out.println("Clean Control");
    	//this.activateControl();
    }
    
    public void setProbeDataRate(String name, int probe_timeout) {
    	Probe probe = dataSource.getProbeByName(name);
    	if(probe == null)
    		return;
    	int milliseconds_each_hour = 3600000;
		Rational probe_rate = new Rational(milliseconds_each_hour,1000);
		try{
			probe_rate = new Rational(milliseconds_each_hour, probe_timeout);
		}catch(Exception e){
			e.printStackTrace();
		}
		probe.setDataRate(probe_rate);
    }
    
    private void loadParametersFromRegistry(){
    	String propsFile = "config/resource_controller.config";
    	Properties props = new Properties();
		try {
			props.load(new FileInputStream(propsFile));
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		cpuComponentID = props.getProperty("CPU_Component_ID");
		memComponentID = props.getProperty("MEM_Component_ID");
		netComponentID = props.getProperty("NET_Component_ID");
		dskComponentID = props.getProperty("DSK_Component_ID");
		jmxComponentID = props.getProperty("JMX_Component_ID");
		IP_Address = props.getProperty("Producer_IP_Address");
		groupId = props.getProperty("Producer_Group");
		providerId = props.getProperty("Producer_Provider");
    }
    
}
