package eu.cloudtm.wpm.sw_probe;

import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import eu.cloudtm.rmi.statistics.CustomMis;
import eu.cloudtm.rmi.statistics.InfinispanStatistics;
import eu.reservoir.monitoring.core.DefaultProbeAttribute;
import eu.reservoir.monitoring.core.DefaultProbeValue;
import eu.reservoir.monitoring.core.ProbeAttributeType;

/**
 * Display amount of free and used memory in the system.
 */
public class InfinispanInfo {
	
	private MBeanServerConnection mbsc = null;
	private String address;
	private int port;
	private String cacheName;
	private String jmxDomainName;
	private String replicationType;
	private String cacheManager;
	private final static long period = 30000; 
    
	public InfinispanInfo(String addr, int port_num,String cache_name,String jmxDomain,String repl,String manager) {
    	address = addr;
    	port = port_num;
    	cacheName = cache_name;
    	jmxDomainName = jmxDomain;
    	replicationType = repl;
    	cacheManager = manager; 
    	connectToJMXServer();
    	
    }

    public InfinispanValue getInfinispanValues(){
    	if(mbsc == null)
    		connectToJMXServer();
    	InfinispanValue inf_value = new InfinispanValue();
    	try{
    		//ObjectName transactionComponent = new ObjectName(jmxDomainName+":type=Cache,name="+ObjectName.quote(cacheName+"("+replicationType+")")+",manager="+ObjectName.quote(cacheManager)+",component=Transactions");
    		ObjectName statisticsComponent = new ObjectName(jmxDomainName+":type=Cache,name="+ObjectName.quote(cacheName+"("+replicationType+")")+",manager="+ObjectName.quote(cacheManager)+",component=TLStatistics");
    		
			Field[] jmx_attributes = InfinispanValue.class.getDeclaredFields();
			for(Field f : jmx_attributes){
				if(f.getType() == double.class){
					Method invoking_method = InfinispanValue.class.getMethod("set"+f.getName(),double.class);
					invoking_method.invoke(inf_value,(Double)mbsc.getAttribute(statisticsComponent,f.getName()));
				}else if(f.getType() == long.class){
					Method invoking_method = InfinispanValue.class.getMethod("set"+f.getName(),long.class);
					invoking_method.invoke(inf_value,(Long)mbsc.getAttribute(statisticsComponent,f.getName()));
				}else if(f.getType() == int.class){
					Method invoking_method = InfinispanValue.class.getMethod("set"+f.getName(),int.class);
					invoking_method.invoke(inf_value,(Integer)mbsc.getAttribute(statisticsComponent,f.getName()));
				}else if(f.getType() == String.class){
					Method invoking_method = InfinispanValue.class.getMethod("set"+f.getName(),String.class);
					invoking_method.invoke(inf_value,(String)mbsc.getAttribute(statisticsComponent,f.getName()));
				}else if(f.getType() == Map.class){
					Method invoking_method = InfinispanValue.class.getMethod("set"+f.getName(),Map.class);
					invoking_method.invoke(inf_value,(Map<Long,Long>)mbsc.getAttribute(statisticsComponent,f.getName()));
				}else
					throw new RuntimeException("Format not supported!");
			}
			
    		/*
    		//CopyOnWriteArrayList<InfinispanStatistics> statis =  (CopyOnWriteArrayList<InfinispanStatistics>)mbsc.getAttribute(statisticsComponent,"Statistics");
    		inf_value.setLocksAcquisitionRate((Double)mbsc.getAttribute(statisticsComponent,"LocksAcquisitionRate"));
    		inf_value.setRollbacks((Long)mbsc.getAttribute(statisticsComponent,"Rollbacks"));
    		inf_value.setLocalExecNoCont((Long)mbsc.getAttribute(statisticsComponent,"LocalExecNoCont"));
    		inf_value.setRollbackCommandCost((Long)mbsc.getAttribute(statisticsComponent,"RollbackCommandCost"));
    		inf_value.setApplicationContentionFactor((Double)mbsc.getAttribute(statisticsComponent,"ApplicationContentionFactor"));
    		inf_value.setMaxReplayTime((Double)mbsc.getAttribute(statisticsComponent,"MaxReplayTime"));
    		inf_value.setCommittedTransactionsWritePercentage((Double)mbsc.getAttribute(statisticsComponent,"CommittedTransactionsWritePercentage"));
    		inf_value.setThroughput((Long)mbsc.getAttribute(statisticsComponent,"Throughput"));
    		inf_value.setWriteTransactionTotalExecutionTime((Long)mbsc.getAttribute(statisticsComponent,"WriteTransactionTotalExecutionTime"));
    		inf_value.setCommitCommandCost((Long)mbsc.getAttribute(statisticsComponent,"CommitCommandCost"));
    		inf_value.setTransactionsWritePercentage((Double)mbsc.getAttribute(statisticsComponent,"TransactionsWritePercentage"));
    		inf_value.setCommitCommandSize((Long)mbsc.getAttribute(statisticsComponent,"CommitCommandSize"));
    		inf_value.setTransactionsArrivalRate((Double)mbsc.getAttribute(statisticsComponent,"TransactionsArrivalRate"));
    		inf_value.setLocalContentionProbability((Double)mbsc.getAttribute(statisticsComponent,"LocalContentionProbability"));
    		inf_value.setLockWaitingTime((Long)mbsc.getAttribute(statisticsComponent,"LockWaitingTime"));
    		inf_value.setReadOnlyTransactionExecutionTime((Long)mbsc.getAttribute(statisticsComponent,"ReadOnlyTransactionExecutionTime"));
    		inf_value.setWriteTransactionLocalExecutionTime((Long)mbsc.getAttribute(statisticsComponent,"WriteTransactionLocalExecutionTime"));
    		inf_value.setPerTransactionAcquiredLocks((Double)mbsc.getAttribute(statisticsComponent,"PerTransactionAcquiredLocks"));
    		inf_value.setPrepareCommandSize((Long)mbsc.getAttribute(statisticsComponent,"PrepareCommandSize"));
    		inf_value.setAvgReplayTime((Double)mbsc.getAttribute(statisticsComponent,"AvgReplayTime"));
    		
    		//mbsc.invoke(statisticsComponent, "resetStatistics", new Object[0], new String[0]);
    		*/
    		
    	}catch (IOException e) {
    		e.printStackTrace();
		} catch (MalformedObjectNameException e) {
			e.printStackTrace();
		} catch (NullPointerException e) {
			e.printStackTrace();
		} catch (AttributeNotFoundException e) {
			e.printStackTrace();
		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (MBeanException e) {
			e.printStackTrace();
		} catch (ReflectionException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
    	
		return inf_value;
    }
    
    public InfinispanTopK getInfinispanTopK(){
    	if(mbsc == null)
    		connectToJMXServer();
    	InfinispanTopK inf_value = new InfinispanTopK();
    	try{
    		ObjectName streamComponent = new ObjectName(jmxDomainName+":type=Cache,name="+ObjectName.quote(cacheName+"("+replicationType+")")+",manager="+ObjectName.quote(cacheManager)+",component=StreamLibStatistics");
    		
			Field[] jmx_attributes = InfinispanTopK.class.getDeclaredFields();
			for(Field f : jmx_attributes){
				if(f.getType() == double.class){
					Method invoking_method = InfinispanTopK.class.getMethod("set"+f.getName(),double.class);
					invoking_method.invoke(inf_value,(Double)mbsc.getAttribute(streamComponent,f.getName()));
				}else if(f.getType() == long.class){
					Method invoking_method = InfinispanTopK.class.getMethod("set"+f.getName(),long.class);
					invoking_method.invoke(inf_value,(Long)mbsc.getAttribute(streamComponent,f.getName()));
				}else if(f.getType() == int.class){
					Method invoking_method = InfinispanTopK.class.getMethod("set"+f.getName(),int.class);
					invoking_method.invoke(inf_value,(Integer)mbsc.getAttribute(streamComponent,f.getName()));
				}else if(f.getType() == String.class){
					Method invoking_method = InfinispanTopK.class.getMethod("set"+f.getName(),String.class);
					invoking_method.invoke(inf_value,(String)mbsc.getAttribute(streamComponent,f.getName()));
				}else if(f.getType() == Map.class){
					Method invoking_method = InfinispanTopK.class.getMethod("set"+f.getName(),Map.class);
					invoking_method.invoke(inf_value,(Map<Long,Long>)mbsc.getAttribute(streamComponent,f.getName()));
				}else
					throw new RuntimeException("Format not supported!");
			}
    	}catch (IOException e) {
    		e.printStackTrace();
		} catch (MalformedObjectNameException e) {
			e.printStackTrace();
		} catch (NullPointerException e) {
			e.printStackTrace();
		} catch (AttributeNotFoundException e) {
			e.printStackTrace();
		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (MBeanException e) {
			e.printStackTrace();
		} catch (ReflectionException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
    	
		return inf_value;
    }
    
    private void connectToJMXServer(){
    	JMXServiceURL url = null;
    	JMXConnector jmxc = null;
    	try {
			url = new JMXServiceURL("service:jmx:rmi:///jndi/rmi://"+address+":"+port+"/jmxrmi");
			jmxc = JMXConnectorFactory.connect(url, null);
			mbsc = jmxc.getMBeanServerConnection();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
    }
    
    private Long getRate(Long attribute,Long samplingPeriod){
    	return attribute / samplingPeriod;
    }
}

