package eu.cloudtm.wpm.sw_probe;

import java.io.FileInputStream;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Map;
import java.util.Properties;

import eu.cloudtm.resources.MonitorableResources;
import eu.reservoir.monitoring.core.AbstractProbe;
import eu.reservoir.monitoring.core.DefaultProbeAttribute;
import eu.reservoir.monitoring.core.DefaultProbeValue;
import eu.reservoir.monitoring.core.Probe;
import eu.reservoir.monitoring.core.ProbeAttributeType;
import eu.reservoir.monitoring.core.ProbeMeasurement;
import eu.reservoir.monitoring.core.ProbeValue;
import eu.reservoir.monitoring.core.ProducerMeasurement;
import eu.reservoir.monitoring.core.Rational;
import eu.reservoir.monitoring.core.TypeException;


public class InfinispanResourceProbe extends AbstractProbe implements Probe {
	static InfinispanInfo monitored_infinispan;
	static String addr;
	static int port_num; 
	static String cache_name;
	static String jmxDomain;
	static String replicationType;
	static String cacheManager;
	
	//probe_timeout in millisecond
	public InfinispanResourceProbe(String name,int probe_timeout){
		setName(name);
		//Logical group of VM
		//ID gr_id = new ID(group_id);
		//setGroupID(gr_id);
		//Specified in measurements per hour
		int milliseconds_each_hour = 3600000;
		Rational probe_rate = new Rational(milliseconds_each_hour,1000);
		try{
			probe_rate = new Rational(milliseconds_each_hour, probe_timeout);
		}catch(Exception e){
			e.printStackTrace();
		}
        setDataRate(probe_rate);
        //retrive JMX and infinispan parameters
        loadParametersFromRegistry();
        monitored_infinispan = new InfinispanInfo(addr, port_num, cache_name, jmxDomain,replicationType,cacheManager);
        setProbeAttributes(cache_name);
	}
	
	private void setProbeAttributes(String cache_name){
		int attributeKey = 0;
		Field[] jmx_attributes = InfinispanValue.class.getDeclaredFields();
		for(Field f : jmx_attributes){
			if(f.getType() == double.class){
				addProbeAttribute(new DefaultProbeAttribute(attributeKey++, f.getName(), ProbeAttributeType.DOUBLE, ""));
			}else if(f.getType() == long.class){
				addProbeAttribute(new DefaultProbeAttribute(attributeKey++, f.getName(), ProbeAttributeType.LONG, ""));
			}else if(f.getType() == int.class){
				addProbeAttribute(new DefaultProbeAttribute(attributeKey++, f.getName(), ProbeAttributeType.INTEGER, ""));
			}else if(f.getType() == String.class){
				addProbeAttribute(new DefaultProbeAttribute(attributeKey++, f.getName(), ProbeAttributeType.STRING, ""));
			}else if(f.getType() == Map.class){
				addProbeAttribute(new DefaultProbeAttribute(attributeKey++, f.getName(), ProbeAttributeType.MAP, ""));
			}else
				throw new RuntimeException("Format not supported!");
		}
		jmx_attributes = InfinispanTopK.class.getDeclaredFields();
		for(Field f : jmx_attributes){
			if(f.getType() == double.class){
				addProbeAttribute(new DefaultProbeAttribute(attributeKey++, f.getName(), ProbeAttributeType.DOUBLE, ""));
			}else if(f.getType() == long.class){
				addProbeAttribute(new DefaultProbeAttribute(attributeKey++, f.getName(), ProbeAttributeType.LONG, ""));
			}else if(f.getType() == int.class){
				addProbeAttribute(new DefaultProbeAttribute(attributeKey++, f.getName(), ProbeAttributeType.INTEGER, ""));
			}else if(f.getType() == String.class){
				addProbeAttribute(new DefaultProbeAttribute(attributeKey++, f.getName(), ProbeAttributeType.STRING, ""));
			}else if(f.getType() == Map.class){
				addProbeAttribute(new DefaultProbeAttribute(attributeKey++, f.getName(), ProbeAttributeType.MAP, ""));
			}else
				throw new RuntimeException("Format not supported!");
		}
	}
	
	public ProbeMeasurement collect() {
		System.out.println("Start collecting at: "+System.currentTimeMillis());
		// list of proble values
		ArrayList<ProbeValue> list = new ArrayList<ProbeValue>();
		int attributeKey = 0;
		//collect data from infinispan using JMX
		try {
			InfinispanValue inf_value = monitored_infinispan.getInfinispanValues();
			Field[] jmx_attributes = InfinispanValue.class.getDeclaredFields();
			for(Field f : jmx_attributes){
				Method invoking_method = InfinispanValue.class.getMethod("get"+f.getName());
				Object d = invoking_method.invoke(inf_value);
				list.add(new DefaultProbeValue(attributeKey++, d));
			}
			InfinispanTopK inf_tk = monitored_infinispan.getInfinispanTopK();
			Field[] jmx_attributes_tk = InfinispanTopK.class.getDeclaredFields();
			for(Field f : jmx_attributes_tk){
				Method invoking_method = InfinispanTopK.class.getMethod("get"+f.getName());
				Object d = invoking_method.invoke(inf_tk);
				list.add(new DefaultProbeValue(attributeKey++, d));
			}
		} catch (TypeException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
		
		return new ProducerMeasurement(this, list, MonitorableResources.JMX.toString());
	}

	private static void loadParametersFromRegistry(){
    	String propsFile = "config/infinispan_probe.config";
    	Properties props = new Properties();
		try {
			props.load(new FileInputStream(propsFile));
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		addr = props.getProperty("Infinispan_IP_Address");
		port_num = Integer.parseInt(props.getProperty("Infinispan_JMX_port_number")); 
		cache_name = props.getProperty("Cache_name");
		jmxDomain = props.getProperty("JMXDomain");
		replicationType = props.getProperty("Replication_type");
		cacheManager = props.getProperty("Cache_Manager");
    }
}